@@ -20,6 +20,27 @@
 
 extern SDL_Renderer *mrc;
 
+/** Check if a theme by @name is an old classic theme from LBreakout2
+ * which is the case if no theme.ini is present or oldTheme is 1. */
+bool Theme::isOldTheme(const string &name) {
+	string path;
+
+	if (name[0] == '~')
+		path = getHomeDir() + "/" + CONFIGDIR + "/themes/" + name.substr(1);
+	else
+		path = string(DATADIR) + "/themes/" + name;
+
+	if (!fileExists(path + "/theme.ini"))
+		return true;
+	FileParser fp(path + "/theme.ini");
+	uint old = 0;
+	fp.get("oldTheme",old);
+	if (old)
+		return true;
+
+	return false;
+}
+
 /** Load resources and scale if necessary using bricks screen height.
  * Whatever is missing: Fall back to Standard theme. */
 void Theme::load(string name, uint screenWidth, uint screenHeight,
@@ -41,15 +62,15 @@ void Theme::load(string name, uint screenWidth, uint screenHeight,
 	_loginfo("Loading theme %s\n",path.c_str());
 
 	/* set default config values for old themes */
-	oldTheme = true;
+	oldTheme = 1;
 	title = "unknown";
 	author = "unknown";
 	version= "v?.??";
 	brickFileWidth = 40;
 	brickFileHeight = 20;
 	shadowOffset = 10;
-	fontColorNormal = {255,255,255,255};
-	fontColorHighlight = {255,220,0,255};
+	fontColorNormal = {255,220,0,255};
+	fontColorHighlight = {255,255,255,255};
 	shotFrameNum = 4;
 	shotAnimDelay = 200;
 	weaponFrameNum = 4;
@@ -70,13 +91,14 @@ void Theme::load(string name, uint screenWidth, uint screenHeight,
 	menuFontNormalSize = 18;
 	menuFontFocusName = "fnormal.otf";
 	menuFontFocusSize = 21;
-	menuFontColorNormal = {255,255,255,255};
-	menuFontColorFocus  = {255,220,0,255};
+	menuFontColorFocus = {255,255,255,255};
+	menuFontColorNormal  = {255,220,0,255};
 
 	/* load theme values */
 	if (fileExists(path + "/theme.ini")) {
-		oldTheme = false;
 		FileParser fp(path + "/theme.ini");
+		oldTheme = 0;
+		fp.get("oldTheme",oldTheme);
 		fp.get("title",title);
 		fp.get("author",author);
 		fp.get("version",version);
@@ -126,6 +148,10 @@ void Theme::load(string name, uint screenWidth, uint screenHeight,
 	boardWidth = 4.33 * brickScreenWidth;
 
 	/* load standard values for fallback */
+	if (oldTheme)
+		stdPath = string(DATADIR) + "/themes/Classic";
+	else
+		stdPath = string(DATADIR) + "/themes/Standard";
 	FileParser stdSettings(stdPath + "/theme.ini");
 	int sbfw, sbfh; /* standard brick file width/height */
 	stdSettings.get("brickWidth",sbfw);
@@ -351,10 +377,7 @@ void Theme::load(string name, uint screenWidth, uint screenHeight,
 
 	/* warp symbol is brick size */
 	/* ignore old warp icon as it's geometry sucks big time */
-	if (!oldTheme)
-		warpIcon.load(testRc(path,"warp.png"));
-	else
-		warpIcon.load(stdPath + "/warp.png");
+	warpIcon.load(testRc(path,"warp.png"));
 	warpIcon.scale(brickScreenWidth,brickScreenHeight);
 
 	/* explosions are square, scaled according to brick ratio */
