@@ -27,7 +27,7 @@ extern SDL_Renderer *mrc;
 extern int last_ball_brick_reflect_x;
 
 View::View(Config &cfg, ClientGame &_cg)
-	: config(cfg), mw(NULL), editor(theme,mixer),
+	: config(cfg), mw(NULL), editor(theme,mixer), layout(VL_STANDARD),
 	  curMenu(NULL), graphicsMenu(NULL), resumeMenuItem(NULL),
 	  selectDlg(theme, mixer), lblCredits1(true), lblCredits2(true),
 	  cgame(_cg), quitReceived(false),
@@ -110,58 +110,80 @@ void View::init(string t, uint r)
 	}
 	_loginfo("Using display %d\n",cpdix);
 
-	/* determine resolution and scale factor */
-	int ww, wh; /* size of sdl window */
-	int sw, sh; /* either screen or viewport size, is used to scale assets */
-	if (r == 0) {
-		/* fullscreen is tricky... might also be 16:10,4:3,...
-		 * set largest possible 16:9 viewport centered */
-		SDL_DisplayMode mode;
-		SDL_GetCurrentDisplayMode(cpdix,&mode);
-		/* TEST  mode.w = 1600; mode.h = 1200; */
-		ww = mode.w;
-		wh = mode.h;
-		sw = mode.w;
-		sh = mode.w/16*9;
-		if (sh != wh) {
-			if (sh < wh) { /* e.g. 4:3 */
-				viewport.x = 0;
-				viewport.y = (wh - sh)/2;
-			} else { /* e.g. 21:9 */
-				sh = wh;
-				sw = sh / 9 * 16;
-				viewport.x = (ww - sw)/2;
-				viewport.y = 0;
-			}
-			viewport.w = sw;
-			viewport.h = sh;
-			_loginfo("Fullscreen resolution %dx%d not 16:9\n",ww,wh);
-			_loginfo("  Using viewport x=%d,y=%d,w=%d,h=%d\n",
-					viewport.x, viewport.y, viewport.w, viewport.h);
-		} else
-			_loginfo("Using fullscreen, resolution %dx%d\n",ww,wh);
+	/* determine whether it is an old classic theme and decide layout */
+	int ratioX, ratioY; /* resolution ratio */
+	if (Theme::isOldTheme(t)) {
+		/* retro theme, use 4:3 */
+		layout = VL_CLASSIC;
+		ratioX = 4;
+		ratioY = 3;
+		_loginfo("Classic theme, using 4:3 layout\n");
 	} else {
-		wh = sh = r;
-		ww = sw = sh * 16 / 9;
-		_loginfo("Using window mode, size %dx%d\n",ww,wh);
+		/* modern theme, use 16:9 */
+		layout = VL_STANDARD;
+		ratioX = 16;
+		ratioY = 9;
+		_loginfo("Modern theme, using 16:9 layout\n");
 	}
 
-	/* depending on screen ratio we don't fit all of the screen with bricks */
-	brickScreenHeight = sh / MAPHEIGHT;
+	/* reset viewport */
+	viewport.x = 0;
+	viewport.y = 0;
+	viewport.w = 0;
+	viewport.h = 0;
+
+	/* determine viewport */
+	int ww, wh; /* size of sdl window */
+	if (r == 0) { /* fullscreen */
+		SDL_DisplayMode mode;
+		SDL_GetCurrentDisplayMode(cpdix,&mode);
+		ww = mode.w;
+		wh = mode.h;
+		viewport.w = ww; /* try optimal width */
+		viewport.h = ww/ratioX*ratioY;
+		if (viewport.h > wh) {
+			/* doesn't fit, use optimal height */
+			viewport.h = wh;
+			viewport.w = wh/ratioY*ratioX;
+		}
+		_loginfo("Using fullscreen, resolution %dx%d\n",ww,wh);
+	} else { /* window */
+		wh = r;
+		ww = wh * ratioX / ratioY;
+		viewport.w = ww;
+		viewport.h = wh;
+		_loginfo("Using window mode, size %dx%d\n",ww,wh);
+		/* TEST ww = 1280; wh = 720; */
+	}
+
+	/* determine scale factor and adjust viewport if needed */
+	brickScreenHeight = viewport.h / MAPHEIGHT;
 	scaleFactor = brickScreenHeight * 100 / VG_BRICKHEIGHT;
 	brickScreenWidth = v2s(VG_BRICKWIDTH);
-	brickAreaWidth = brickScreenWidth * MAPHEIGHT;
-	brickAreaHeight = brickScreenHeight * MAPHEIGHT;
 	_loginfo("Scale factor x100: %d\n",scaleFactor);
 	_loginfo("Brick screen size: %dx%d\n",brickScreenWidth,brickScreenHeight);
 
+	/* adjust viewport size if needed to fit bricks perfectly */
+	if (brickScreenHeight * MAPHEIGHT < (uint)viewport.h)
+		viewport.h = brickScreenHeight*MAPHEIGHT;
+	if (layout == VL_CLASSIC && brickScreenWidth * MAPWIDTH < (uint)viewport.w)
+		viewport.w = brickScreenWidth * MAPWIDTH;
+	/* we don't adjust for standard 16:9 layout as it is only a minimal error */
+
+	/* adjust viewport position */
+	viewport.x = (ww - viewport.w) / 2;
+	viewport.y = (wh - viewport.h) / 2;
+
+	_loginfo("Using viewport x=%d,y=%d,w=%d,h=%d\n",
+			viewport.x, viewport.y, viewport.w, viewport.h);
+
 	/* (re)create main window */
 	if (mw)
 		delete mw;
 	mw = new MainWindow("LBreakoutHD", ww, wh, (r==0) );
 
 	/* load theme (scaled if necessary) */
-	theme.load(t, sw, sh, brickScreenWidth, brickScreenHeight, config.antialiasing);
+	theme.load(t, viewport.w, viewport.h, brickScreenWidth, brickScreenHeight, config.antialiasing);
 	weaponFrameCounter.init(theme.weaponFrameNum, theme.weaponAnimDelay);
 	shotFrameCounter.init(theme.shotFrameNum, theme.shotAnimDelay);
 	shineFrameCounter.init(theme.shineFrameNum, theme.shineAnimDelay);
@@ -179,20 +201,32 @@ void View::init(string t, uint r)
 	lblInfo.setBorder(2);
 
 	/* create render images and positions */
-	imgBackground.create(sw,sh);
+	imgBackground.create(viewport.w,viewport.h);
 	imgBackground.setBlendMode(0);
 	curWallpaperId = rand() % theme.numWallpapers;
-	imgScore.create(brickScreenWidth*3, brickScreenHeight);
-	imgScoreX = theme.boardX + (theme.boardWidth - imgScore.getWidth())/2;
-	imgScoreY = brickScreenHeight * 15 + brickScreenHeight/2;
+	if (layout == VL_STANDARD) {
+		imgScore.create(brickScreenWidth*3, brickScreenHeight);
+		imgScoreX = theme.boardX + (theme.boardWidth - imgScore.getWidth())/2;
+		imgScoreY = brickScreenHeight * 15 + brickScreenHeight/2;
+	} else {
+		imgScore.create(4.5*brickScreenWidth, 0.8*brickScreenHeight);
+		imgScoreX = 10.125 * brickScreenWidth;
+		imgScoreY = 0;
+	}
 	/* EDITHEIGHT+1 enough for everything except barrier so make it bigger */
 	imgBricks.create(EDITWIDTH*brickScreenWidth,
 				(EDITHEIGHT+4)*brickScreenHeight);
 	imgBricksX = brickScreenWidth;
 	imgBricksY = brickScreenHeight;
-	imgExtras.create(theme.boardWidth, 4*brickScreenHeight);
-	imgExtrasX = theme.boardX;
-	imgExtrasY = 19*brickScreenHeight;
+	if (layout == VL_STANDARD) {
+		imgExtras.create(theme.boardWidth, 4*brickScreenHeight);
+		imgExtrasX = theme.boardX;
+		imgExtrasY = 19*brickScreenHeight;
+	} else {
+		imgExtras.create(brickScreenWidth, 13*brickScreenHeight);
+		imgExtrasX = (MAP_WIDTH-1)*brickScreenWidth;
+		imgExtrasY = 3.9*brickScreenHeight;
+	}
 	imgFloor.create(EDITWIDTH*brickScreenWidth,brickScreenHeight);
 	imgFloorX = brickScreenWidth;
 	imgFloorY = (MAPHEIGHT-1)*brickScreenHeight;
@@ -202,6 +236,14 @@ void View::init(string t, uint r)
 	SDL_SetRenderTarget(mrc, NULL);
 	warpIconX = (MAPWIDTH - 2)*brickScreenWidth;
 	warpIconY = (MAPHEIGHT - 1)*brickScreenHeight;
+
+	/* clear viewport if not really needed */
+	if (viewport.w == ww && viewport.h == wh) {
+		viewport.x = 0;
+		viewport.y = 0;
+		viewport.w = 0;
+		viewport.h = 0;
+	}
 }
 
 View::~View()
@@ -238,10 +280,9 @@ void View::run()
 	SDL_Event ev;
 	int flags;
 	PaddleInputState pis;
-	Ticks ticks;
-	Ticks renderTicks;
-	int maxDelay, delay = 0;
-	Uint32 ms;
+	HPTicks ticks;
+	int frameDelay;
+	double ms;
 	bool leave = false;
 	bool resumeLater = false;
 	double rx = 0;
@@ -253,12 +294,14 @@ void View::run()
 	fpsStart = SDL_GetTicks();
 	fpsCycles = 0;
 
-	if (config.fps == 1)
-		maxDelay = 5;
-	else if (config.fps == 2)
-		maxDelay = 10;
+	/* FIXME assume render time of ~1ms and use fixed delay as
+	 * variable Delay causes jittering because 1ms steps is too coarse */
+	if (config.fps == 0)
+		frameDelay = 16; /* roughly 60 FPS */
+	else if (config.fps == 1)
+		frameDelay = 7; /* roughly 120 FPS */
 	else
-		maxDelay = 0;
+		frameDelay = 2; /* roughly 300 FPS */
 
 	initTitleLabel();
 	lblInfo.clearText();
@@ -266,12 +309,12 @@ void View::run()
 	renderBackgroundImage();
 	renderBricksImage();
 	renderScoreImage();
-	render();
+	render(true);
+	fade(FADE_IN);
 
 	grabInput(1);
 
 	while (!leave) {
-		renderTicks.reset();
 		flags = 0;
 
 		/* handle events */
@@ -287,6 +330,12 @@ void View::run()
 					gamepad.close();
 					gamepad.open();
 					break;
+				case SDL_SCANCODE_H:
+					if (layout == VL_CLASSIC) {
+						showFinalHiscores();
+						ticks.reset();
+					}
+					break;
 				case SDL_SCANCODE_P:
 					showInfo(_("Pause"),WT_PAUSE);
 					ticks.reset();
@@ -448,8 +497,7 @@ void View::run()
 			if (flags & CGF_NEWLEVEL)
 				if (!(flags & CGF_LIFELOST) && config.speech && (rand()%2))
 					mixer.play((rand()%2)?theme.sVeryGood:theme.sExcellent);
-			dim();
-			ticks.reset();
+			fade();
 			if (!(flags & CGF_LIFELOST)) {
 				initTitleLabel();
 				showWarpIcon = false;
@@ -472,12 +520,20 @@ void View::run()
 		if (flags & CGF_UPDATEINFO)
 			lblInfo.setText(theme.fSmall, cgame.getBonusLevelInfo());
 
+		if ((flags & CGF_NEWLEVEL) || (flags & CGF_RESTARTLEVEL)) {
+			render(true);
+			fade(FADE_IN);
+			ticks.reset();
+		}
+
 		/* handle sounds by accessing game->mod */
 		playSounds();
 
 		/* render */
-		render();
-		SDL_RenderPresent(mrc);
+		if (!leave) { /* don't when leaving to keep dialog for fading effect */
+			render();
+			SDL_RenderPresent(mrc);
+		}
 
 		/* stats */
 		fpsCycles++;
@@ -489,9 +545,7 @@ void View::run()
 		}
 
 		/* limit frame rate */
-		delay = maxDelay - renderTicks.get(true);
-		if (delay > 0)
-			SDL_Delay(delay);
+		SDL_Delay(frameDelay);
 		SDL_FlushEvent(SDL_MOUSEMOTION); /* prevent event loop from dying */
 	}
 
@@ -510,21 +564,28 @@ void View::run()
 	}
 	updateResumeGameTooltip();
 
-	if (!quitReceived)
-		dim();
+	fade();
 	grabInput(0);
 }
 
-/** Render current game state. */
-void View::render()
+/** Render current game state. If @clear is true fill whole screen black first
+ * if viewport is used. */
+void View::render(bool clear)
 {
 	Game *game = cgame.getGameContext(); /* direct lib game context */
 	Paddle *paddle = game->paddles[0]; /* local paddle always at bottom */
 	Extra *extra = 0;
 	Shot *shot = 0;
 
-	if (viewport.w != 0)
+	if (viewport.w != 0) {
+		/* XXX filling whole screen with black should only be needed
+		 * when clear is set but it seems the area outside the viewport
+		 * is not stable so we need to clear everytime when using
+		 * a viewport. */
+		SDL_SetRenderDrawColor(mrc,2,2,2,255);
+		SDL_RenderClear(mrc);
 		SDL_RenderSetViewport(mrc,&viewport);
+	}
 
 	if (cgame.darknessActive()) {
 		SDL_SetRenderDrawColor(mrc,0,0,0,255);
@@ -663,6 +724,10 @@ void View::render()
 					(MAPHEIGHT-1)*brickScreenHeight,sx,sy);
 	}
 
+	/* copy active extras again for classic layout as on right-hand side of frame */
+	if (layout == VL_CLASSIC)
+		imgExtras.copy(imgExtrasX,imgExtrasY);
+
 	/* title and author */
 	if (lblTitleCounter.isRunning()) {
 		double a = 255;
@@ -671,7 +736,7 @@ void View::render()
 		else if (lblTitleCounter.get() >= 4)
 			a = (5-lblTitleCounter.get()) * 255;
 		lblTitle.setAlpha(a);
-		lblTitle.copy((1+MAPWIDTH/2)*brickScreenWidth, mw->getHeight()/2);
+		lblTitle.copy(MAPWIDTH/2*brickScreenWidth, mw->getHeight()/2);
 	}
 
 	/* info for bonus levels */
@@ -691,7 +756,7 @@ void View::render()
 		//theme.fSmall.write(0,theme.fSmall.getLineHeight(),to_string((int)(cgame.getPaddleVelocity()*1000)));
 	}
 
-	if (viewport.w)
+	if (viewport.w != 0)
 		SDL_RenderSetViewport(mrc, NULL);
 }
 
@@ -821,9 +886,20 @@ void View::renderScoreImage()
 {
 	imgScore.fill(0,0,0,0);
 	SDL_SetRenderTarget(mrc, imgScore.getTex());
-	theme.fNormal.setAlign(ALIGN_X_CENTER | ALIGN_Y_CENTER);
-	theme.fNormal.write(imgScore.getWidth()/2, imgScore.getHeight()/2,
-				to_string(cgame.getCurrentPlayer()->getScore()));
+	if (layout == VL_STANDARD) {
+		theme.fNormal.setAlign(ALIGN_X_CENTER | ALIGN_Y_CENTER);
+		theme.fNormal.write(imgScore.getWidth()/2, imgScore.getHeight()/2,
+					to_string(cgame.getCurrentPlayer()->getScore()));
+	} else {
+		/* XXX to simplify the process we render the name as well otherwise
+		 * we'd have to adjust render_background as well */
+		theme.fSmall.setAlign(ALIGN_X_LEFT | ALIGN_Y_CENTER);
+		theme.fSmall.write(0, imgScore.getHeight()/2,
+					cgame.getCurrentPlayer()->getName());
+		theme.fSmall.setAlign(ALIGN_X_RIGHT | ALIGN_Y_CENTER);
+		theme.fSmall.write(imgScore.getWidth()-1, imgScore.getHeight()/2,
+					to_string(cgame.getCurrentPlayer()->getScore()));
+	}
 	SDL_SetRenderTarget(mrc, NULL);
 }
 void View::renderExtrasImage()
@@ -841,51 +917,77 @@ void View::renderExtrasImage()
 	SDL_RenderClear(mrc);
 
 	theme.extras.setAlpha(192);
-	/* energy/explosive/weak balls */
-	if (game->extra_active[EX_METAL])
-		renderActiveExtra(EX_METAL, game->extra_time[EX_METAL], x, y);
-	else if (game->extra_active[EX_EXPL_BALL])
-		renderActiveExtra(EX_EXPL_BALL, game->extra_time[EX_EXPL_BALL], x, y);
-	else if (game->extra_active[EX_WEAK_BALL])
-		renderActiveExtra(EX_WEAK_BALL, game->extra_time[EX_WEAK_BALL], x, y);
-	x += xoff;
-	/* slow/fast balls */
-	if (game->extra_active[EX_SLOW])
-		renderActiveExtra(EX_SLOW, game->extra_time[EX_SLOW], x, y);
-	else if (game->extra_active[EX_FAST])
-		renderActiveExtra(EX_FAST, game->extra_time[EX_FAST], x, y);
-	x += xoff;
-	/* chaotic */
-	if (game->extra_active[EX_CHAOS])
-		renderActiveExtra(EX_CHAOS, game->extra_time[EX_CHAOS], x, y);
-	x = xstart;
-	y += yoff;
-	/* sticky paddle */
-	if (game->paddles[0]->extra_active[EX_SLIME])
-		renderActiveExtra(EX_SLIME, game->paddles[0]->extra_time[EX_SLIME], x, y);
-	x += xoff;
-	/* weapon - paddle */
-	if (game->paddles[0]->extra_active[EX_WEAPON])
-		renderActiveExtra(EX_WEAPON, game->paddles[0]->extra_time[EX_WEAPON], x, y);
-	x += xoff;
-	/* goldshower - paddle */
-	if (game->paddles[0]->extra_active[EX_GOLDSHOWER])
-		renderActiveExtra(EX_GOLDSHOWER, game->paddles[0]->extra_time[EX_GOLDSHOWER], x, y);
-	x = xstart;
-	y += yoff;
-	/* wall - paddle */
-	if (game->paddles[0]->extra_active[EX_WALL])
-		renderActiveExtra(EX_WALL, game->paddles[0]->extra_time[EX_WALL], x, y);
-	x += xoff;
-	/* magnet - paddle */
-	if (game->paddles[0]->extra_active[EX_BONUS_MAGNET])
-		renderActiveExtra(EX_BONUS_MAGNET, game->paddles[0]->extra_time[EX_BONUS_MAGNET], x, y);
-	else if (game->paddles[0]->extra_active[EX_MALUS_MAGNET])
-		renderActiveExtra(EX_MALUS_MAGNET, game->paddles[0]->extra_time[EX_MALUS_MAGNET], x, y);
-	x += xoff;
-	/* ghost - paddle */
-	if (game->paddles[0]->extra_active[EX_GHOST_PADDLE])
-		renderActiveExtra(EX_GHOST_PADDLE, game->paddles[0]->extra_time[EX_GHOST_PADDLE], x, y);
+
+	if (layout == VL_CLASSIC) {
+		/* for classic display at right-hand side of frame */
+		int slots[EX_NUMBER];
+		slots[EX_METAL] = slots[EX_WEAK_BALL] = slots[EX_EXPL_BALL] = 0;
+		slots[EX_FAST] = slots[EX_SLOW] = 1;
+		slots[EX_CHAOS] = 2;
+		slots[EX_SLIME] = 3;
+		slots[EX_WEAPON] = 4;
+		slots[EX_WALL] = 5;
+		slots[EX_GOLDSHOWER] = 6;
+		slots[EX_GHOST_PADDLE] = 7;
+		slots[EX_BONUS_MAGNET] = slots[EX_MALUS_MAGNET] = 8;
+
+		for (int i = 0; i < EX_NUMBER; i++)
+			if (game->extra_active[i])
+				renderActiveExtra(i, game->extra_time[i], 0,
+						1.5*slots[i]*brickScreenHeight);
+		/* this works as there is no multiplayer in lbreakhouthd */
+		for (int i = 0; i < EX_NUMBER; i++)
+			if (game->paddles[0]->extra_active[i])
+				renderActiveExtra(i, game->paddles[0]->extra_time[i], 0,
+						1.5*slots[i]*brickScreenHeight);
+	} else {
+
+		/* energy/explosive/weak balls */
+		if (game->extra_active[EX_METAL])
+			renderActiveExtra(EX_METAL, game->extra_time[EX_METAL], x, y);
+		else if (game->extra_active[EX_EXPL_BALL])
+			renderActiveExtra(EX_EXPL_BALL, game->extra_time[EX_EXPL_BALL], x, y);
+		else if (game->extra_active[EX_WEAK_BALL])
+			renderActiveExtra(EX_WEAK_BALL, game->extra_time[EX_WEAK_BALL], x, y);
+		x += xoff;
+		/* slow/fast balls */
+		if (game->extra_active[EX_SLOW])
+			renderActiveExtra(EX_SLOW, game->extra_time[EX_SLOW], x, y);
+		else if (game->extra_active[EX_FAST])
+			renderActiveExtra(EX_FAST, game->extra_time[EX_FAST], x, y);
+		x += xoff;
+		/* chaotic */
+		if (game->extra_active[EX_CHAOS])
+			renderActiveExtra(EX_CHAOS, game->extra_time[EX_CHAOS], x, y);
+		x = xstart;
+		y += yoff;
+		/* sticky paddle */
+		if (game->paddles[0]->extra_active[EX_SLIME])
+			renderActiveExtra(EX_SLIME, game->paddles[0]->extra_time[EX_SLIME], x, y);
+		x += xoff;
+		/* weapon - paddle */
+		if (game->paddles[0]->extra_active[EX_WEAPON])
+			renderActiveExtra(EX_WEAPON, game->paddles[0]->extra_time[EX_WEAPON], x, y);
+		x += xoff;
+		/* goldshower - paddle */
+		if (game->paddles[0]->extra_active[EX_GOLDSHOWER])
+			renderActiveExtra(EX_GOLDSHOWER, game->paddles[0]->extra_time[EX_GOLDSHOWER], x, y);
+		x = xstart;
+		y += yoff;
+		/* wall - paddle */
+		if (game->paddles[0]->extra_active[EX_WALL])
+			renderActiveExtra(EX_WALL, game->paddles[0]->extra_time[EX_WALL], x, y);
+		x += xoff;
+		/* magnet - paddle */
+		if (game->paddles[0]->extra_active[EX_BONUS_MAGNET])
+			renderActiveExtra(EX_BONUS_MAGNET, game->paddles[0]->extra_time[EX_BONUS_MAGNET], x, y);
+		else if (game->paddles[0]->extra_active[EX_MALUS_MAGNET])
+			renderActiveExtra(EX_MALUS_MAGNET, game->paddles[0]->extra_time[EX_MALUS_MAGNET], x, y);
+		x += xoff;
+		/* ghost - paddle */
+		if (game->paddles[0]->extra_active[EX_GHOST_PADDLE])
+			renderActiveExtra(EX_GHOST_PADDLE, game->paddles[0]->extra_time[EX_GHOST_PADDLE], x, y);
+	}
 
 	theme.extras.clearAlpha();
 	SDL_SetRenderTarget(mrc, NULL);
@@ -902,20 +1004,23 @@ void View::renderActiveExtra(int id, int ms, int x, int y)
 				to_string(ms/1000+1));
 }
 
-/* Dim screen to black. As game engine is already set for new state
- * we cannot use render() but need current screen state. */
-void View::dim()
+/* Fade in or out current screen content. For fading in
+ * SDL_RenderPresent must not have been called yet. */
+void View::fade(int style)
 {
 	Image img;
 	img.createFromScreen();
 
-	for (uint8_t a = 250; a > 0; a -= 10) {
+	for (uint8_t alpha = 255; alpha > 0; alpha -= 5) {
 		SDL_SetRenderDrawColor(mrc,0,0,0,255);
 		SDL_RenderClear(mrc);
-		img.setAlpha(a);
+		if (style == FADE_OUT)
+			img.setAlpha(alpha);
+		else
+			img.setAlpha(255 - alpha);
 		img.copy();
 		SDL_RenderPresent(mrc);
-		SDL_Delay(10);
+		SDL_Delay(5);
 	}
 
 	/* make sure no input is given yet for next state */
@@ -1143,9 +1248,9 @@ void View::playSounds()
 
 void View::createMenus()
 {
-	Menu *mNewGame, *mOptions, *mAudio, *mGraphics, *mControls, *mAdv, *mEditor;
+	Menu *mNewGame, *mOptions, *mAudio, *mGraphics, *mControls, *mMouse, *mAdv, *mEditor;
 	const char *diffNames[] = {_("Kids"),_("Very Easy"),_("Easy"),_("Medium"),_("Hard") } ;
-	const char *fpsLimitNames[] = {_("No Limit"),_("200 FPS"),_("100 FPS") } ;
+	const char *fpsLimitNames[] = {_("60 FPS"),_("120 FPS"),_("300 FPS") } ;
 	const int bufSizes[] = { 256, 512, 1024, 2048, 4096 };
 	const int channelNums[] = { 8, 16, 32 };
 
@@ -1155,6 +1260,8 @@ void View::createMenus()
 	MenuItem::fFocus = &theme.fMenuFocus;
 	MenuItem::fTooltip = &theme.fSmall;
 	MenuItem::tooltipWidth = 0.3 * theme.menuBackground.getWidth();
+	MenuItem::vpX = viewport.x;
+	MenuItem::vpY = viewport.y;
 
 	rootMenu.reset(); /* delete any old menu ... */
 
@@ -1165,6 +1272,7 @@ void View::createMenus()
 	mGraphics = new Menu(theme);
 	graphicsMenu = mGraphics; /* needed to return after mode/theme change */
 	mControls = new Menu(theme);
+	mMouse = new Menu(theme);
 	mAdv = new Menu(theme);
 	mEditor = new Menu(theme);
 
@@ -1206,19 +1314,11 @@ void View::createMenus()
 	mControls->add(new MenuItemKey(_("Idle Return"),
 			_("Return all idle balls (no effective brick hits for some time) to the paddle."),
 			config.k_return));
-	mControls->add(new MenuItemSep());
 	mControls->add(new MenuItemRange(_("Key Speed"),
 			_("The higher the value the faster the paddle moves by keys."),
 			AID_ADJUSTKEYSPEED,config.i_key_speed,100,1000,50));
-	mControls->add(new MenuItemList(_("Mouse Input"),
-			_("'Absolute' uses absolute mouse position. No fine tuning possible and you might experience delay (if moving too far to the right you'll need to come back to the paddle position first) but it will be more accurate for slow movements.\n'Relative' allows modifying paddle speed but might get inaccurate for slow movements with high resolution and high frame rates."),
-			AID_NONE,config.rel_motion,_("Absolute"),_("Relative")));
-	mControls->add(new MenuItemRange(_("Motion Modifier"),
-			_("Adjust mouse sensitivity. (relative motion only)"),
-			AID_NONE,config.motion_mod,40,160,10));
-	mControls->add(new MenuItemSwitch(_("Invert Motion"),
-			_("Invert mouse motion if needed. (relative motion only)"),
-			AID_NONE,config.invert));
+	mControls->add(new MenuItemSep());
+	mControls->add(new MenuItemSub(_("Mouse Options"),mMouse));
 	mControls->add(new MenuItemSep());
 	mControls->add(new MenuItemList(_("Gamepad"),
 			_("Enable/disable gamepad support. You can press F5 during a game to (re)connect. Buttons cannot be configured yet."),
@@ -1226,8 +1326,20 @@ void View::createMenus()
 	mControls->add(new MenuItemSep());
 	mControls->add(new MenuItemBack(mOptions));
 
+	mMouse->add(new MenuItemList(_("Mouse Input"),
+			_("'Absolute' uses absolute mouse position. No fine tuning possible and you might experience delay (if moving too far to the right you'll need to come back to the paddle position first) but it will be more accurate for slow movements.\n'Relative' allows modifying paddle speed but might get inaccurate for slow movements with high resolution and high frame rates."),
+			AID_NONE,config.rel_motion,_("Absolute"),_("Relative")));
+	mMouse->add(new MenuItemRange(_("Motion Modifier"),
+			_("Adjust mouse sensitivity. (relative motion only)"),
+			AID_NONE,config.motion_mod,40,160,10));
+	mMouse->add(new MenuItemSwitch(_("Invert Motion"),
+			_("Invert mouse motion if needed. (relative motion only)"),
+			AID_NONE,config.invert));
+	mMouse->add(new MenuItemSep());
+	mMouse->add(new MenuItemBack(mControls));
+
 	mGraphics->add(new MenuItemList(_("Theme"),
-			_("Select theme. (not applied yet)"),
+			_("'Standard' is the default HD theme.\n'Classic' is the default retro LBreakout2 theme.\n\nYou can download more retro themes from https://lgames.sf.net/LBreakout2/themes.php\n\n(not applied yet)"),
 			AID_NONE,config.theme_id,themeNames));
 	mGraphics->add(new MenuItemList(_("Mode"),
 			_("Select mode. (not applied yet)"),
@@ -1239,7 +1351,7 @@ void View::createMenus()
 			_("Apply the above settings."),AID_APPLYTHEMEMODE));
 	mGraphics->add(new MenuItemSep());
 	mGraphics->add(new MenuItemList(_("Frame Limit"),
-			_("Maximum number of frames per second.\nBe careful: The higher the limit the more insensitive your mouse might become to slow movements (because relative motion is used and program cycles are shorter).\n200 FPS should be a good value."),
+			_("Maximum number of frames per second.\nBe careful: The higher the limit the more insensitive your mouse might become to slow movements if relative motion is used as program cycles get shorter.\n120 FPS should be a good value."),
 			AID_NONE,config.fps,fpsLimitNames,3));
 	mGraphics->add(new MenuItemSep());
 	mGraphics->add(new MenuItemBack(mOptions));
@@ -1356,7 +1468,7 @@ void View::handleEditor(int type)
 					editor.getCurrentLevel()->author,
 					editor.getCurrentLevel()->bricks,
 					editor.getCurrentLevel()->extras) == 0) {
-				dim();
+				fade();
 				run();
 			}
 		} else
@@ -1376,6 +1488,7 @@ void View::runMenu()
 	curMenu = rootMenu.get();
 	curMenu->resetSelection();
 	renderMenu();
+	fade(FADE_IN);
 
 	while (!quitReceived) {
 		/* handle events */
@@ -1470,15 +1583,19 @@ void View::runMenu()
 				break;
 			case AID_RESUME:
 				if (resumeGame()) {
-					dim();
+					fade(FADE_OUT);
 					run();
+					renderMenu();
+					fade(FADE_IN);
 					ticks.reset();
 				}
 				break;
 			case AID_STARTORIGINAL:
 				cgame.init("LBreakoutHD");
-				dim();
+				fade(FADE_OUT);
 				run();
+				renderMenu();
+				fade(FADE_IN);
 				ticks.reset();
 				break;
 			case AID_STARTCUSTOM:
@@ -1487,11 +1604,14 @@ void View::runMenu()
 				if (selectDlg.run()) {
 					/* if FREAKOUT is started again change
 					 * seed to get different order of levels */
-					if (selectDlg.get() == TOURNAMENT)
+					if (selectDlg.get() == TOURNAMENT ||
+							selectDlg.get() == RANDOM20)
 						config.freakout_seed = rand();
 					cgame.init(selectDlg.get());
-					dim();
+					fade(FADE_OUT);
 					run();
+					renderMenu();
+					fade(FADE_IN);
 					ticks.reset();
 				} else if (selectDlg.quitRcvd())
 					quitReceived = true;
@@ -1506,7 +1626,7 @@ void View::runMenu()
 			case AID_EDITNEWSET:
 			case AID_EDITCUSTOM:
 				handleEditor(aid);
-				dim();
+				fade();
 				ticks.reset();
 				break;
 			}
@@ -1519,7 +1639,7 @@ void View::runMenu()
 			SDL_Delay(10);
 		SDL_FlushEvent(SDL_MOUSEMOTION); /* prevent event loop from dying */
 	}
-	dim();
+	fade();
 
 	/* clear events for menu loop */
 	waitForInputRelease();
@@ -1527,11 +1647,31 @@ void View::runMenu()
 
 void View::renderMenu()
 {
+	int lx, ly; /* label position */
+	if (viewport.w != 0) {
+		SDL_SetRenderDrawColor(mrc,2,2,2,255);
+		SDL_RenderClear(mrc);
+		SDL_RenderSetViewport(mrc,&viewport);
+
+		lx = viewport.w - 2;
+		ly = viewport.h - 2;
+	} else {
+		lx = mw->getWidth() - 2;
+		ly = mw->getHeight() - 2;
+	}
+
 	theme.menuBackground.copy();
-	lblCredits1.copy(mw->getWidth()-2,mw->getHeight(),
-				ALIGN_X_RIGHT | ALIGN_Y_BOTTOM);
-	lblCredits2.copy(mw->getWidth()-2,mw->getHeight() - theme.fSmall.getLineHeight(),
+	lblCredits1.copy(lx, ly, ALIGN_X_RIGHT | ALIGN_Y_BOTTOM);
+	lblCredits2.copy(lx, ly - theme.fSmall.getLineHeight(),
 				ALIGN_X_RIGHT | ALIGN_Y_BOTTOM);
+
+	/* add a hint about pressing h for highscores */
+	if (layout == VL_CLASSIC) {
+		theme.fSmall.setAlign(ALIGN_X_CENTER | ALIGN_Y_CENTER);
+		theme.fSmall.write(v2s(517),v2s(330),
+				_("Press H during game for highscores."));
+	}
+
 	curMenu->render();
 
 	/* XXX don't use cursor yet, all the different event loops are
@@ -1539,6 +1679,9 @@ void View::renderMenu()
 	int cx, cy;
 	SDL_GetMouseState(&cx, &cy);
 	theme.cursor.copy(cx,cy);*/
+
+	if (viewport.w != 0)
+		SDL_RenderSetViewport(mrc, NULL);
 }
 
 void View::grabInput(int grab)
@@ -1816,9 +1959,9 @@ void View::showHelp()
 		"number of lives, ball speed, paddle size and score.\n\n"
 		"While playing you can press\n"
 		"  - ESC to leave the game (you'll have to confirm)\n"
-		"  - w to warp to next level (if enough bricks were destroyed)\n"
-		"  - p to pause/unpause the game\n"
-		"  - d to destroy a brick (workaround for bad level design)\n\n"
+		"  - W to warp to next level (if enough bricks were destroyed)\n"
+		"  - P to pause/unpause the game\n"
+		"  - D to destroy a brick (workaround for bad level design)\n\n"
 		"If you leave a game, you can resume it later (no hiscores entry yet).\n\n"
 		"If you lose all lives, you can buy a continue (score set to 0, initial number of lives restored). "
 		"If you don't buy a continue, the game is over (hiscores entry is checked) and can no longer be resumed. "
@@ -1899,8 +2042,16 @@ void View::runBrickDestroyDlg()
 	int selx = -1, sely = -1;
 	int texty = (EDITHEIGHT+2)*brickScreenHeight;
 
-	imgBackground.copy();
-	darkenScreen(64);
+	if (viewport.w != 0) {
+		SDL_SetRenderDrawColor(mrc,2,2,2,255);
+		SDL_RenderClear(mrc);
+		SDL_RenderSetViewport(mrc,&viewport);
+	}
+
+	/* just have a black background as background image gets
+	 * wrongly scaled for some reason when using a viewport
+	imgBackground.copy(0,0);
+	darkenScreen(64); */
 	imgBricks.setAlpha(128);
 	imgBricks.copy(imgBricksX,imgBricksY);
 	imgBricks.clearAlpha();
@@ -1927,8 +2078,8 @@ void View::runBrickDestroyDlg()
 			if (ev.type == SDL_KEYUP)
 				leave = true;
 			if (ev.type == SDL_MOUSEBUTTONUP && ev.button.button == SDL_BUTTON_LEFT) {
-				selx = ev.button.x / brickScreenWidth;
-				sely = ev.button.y / brickScreenHeight;
+				selx = (ev.button.x - viewport.x) / brickScreenWidth;
+				sely = (ev.button.y - viewport.y) / brickScreenHeight;
 				if (!cgame.destroyBrick(selx,sely))
 					selx = sely = -1; /* no brick found */
 				else
@@ -1938,6 +2089,12 @@ void View::runBrickDestroyDlg()
 		SDL_FlushEvent(SDL_MOUSEMOTION);
 	}
 
+	if (viewport.w != 0) {
+		SDL_RenderSetViewport(mrc, NULL);
+		SDL_SetRenderDrawColor(mrc,2,2,2,255);
+		SDL_RenderClear(mrc);
+	}
+
 	grabInput(1);
 	waitForInputRelease();
 }
